//
//  RefreshGifHeaderLottie.m
//  GenericModules
//
//  Created by mac on 2023/5/23.
//

#import "RefreshGifHeaderLottie.h"
#import <AudioToolbox/AudioToolbox.h>
#import <Lottie/Lottie.h>
#import "AppUIDefine.h"

@interface RefreshGifHeaderLottie ()

@property(nonatomic, strong) LOTAnimationView *loadingView;

@end

@implementation RefreshGifHeaderLottie

- (instancetype)init{
    if (self = [super init]){        
        self.lastUpdatedTimeLabel.hidden = YES;
        self.stateLabel.hidden = YES;
        [self addSubview:self.loadingView];
    }
    return self;
}

- (LOTAnimationView *)loadingView{
    if (!_loadingView){
        _loadingView = [LOTAnimationView animationNamed:@"headerRefresh" inBundle:self.bundle];
        _loadingView.frame = CGRectMake(ScreenWidth/2 - 25, 0, 50, 50);
        _loadingView.loopAnimation = YES;
        _loadingView.contentMode = UIViewContentModeScaleAspectFill;
        _loadingView.animationSpeed = 2.0;
    }
    return _loadingView;
}

- (NSBundle *)bundle {
    static NSBundle *bundle = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        bundle = [NSBundle bundleWithPath:[[NSBundle bundleForClass:[self class]] pathForResource:@"GenericModules" ofType:@"bundle"]];
    });
    return bundle;
}


#pragma mark - innerMethod
- (void)beginRefreshing
{
    if (@available(iOS 10.0, *))
    {
        UIImpactFeedbackGenerator *impactLight = [[UIImpactFeedbackGenerator alloc] initWithStyle:UIImpactFeedbackStyleMedium];
        [impactLight impactOccurred];
    }
    else
    {
        AudioServicesPlaySystemSound(1502);
    }
    [super beginRefreshing];
}

#pragma mark - 监听控件的刷新状态
- (void)setState:(MJRefreshState)state
{
    MJRefreshCheckState;

    switch (state)
    {
        case MJRefreshStateIdle: /// 普通闲置状态
        {
            [self.loadingView stop];
            break;
        }
        case MJRefreshStatePulling:
        {
            break;
        }
        case MJRefreshStateRefreshing:
        {
            self.loadingView.animationProgress = 0;
            [self.loadingView play];
            break;
        }
        default:
            break;
    }
    
}

#pragma mark - 实时监听控件 scrollViewContentOffset
- (void)scrollViewContentOffsetDidChange:(NSDictionary *)change{
    [super scrollViewContentOffsetDidChange:change];
  
    CGPoint point;
    id newVelue = [change valueForKey:NSKeyValueChangeNewKey];
    [(NSValue *)newVelue getValue:&point];
    
    self.loadingView.hidden = !(self.pullingPercent);
    CGFloat progress = point.y / (ScreenHeight/3.0);
    if (self.state != MJRefreshStateRefreshing)
    {
        self.loadingView.animationProgress = -progress;
    }
}

@end
